@@ -13,6 +13,11 @@ class MatrixBase
 public:
     virtual ~MatrixBase(){};
 
+    bool is_square() const
+    {
+        return ncols() == nrows();
+    }
+
     // Below methods should be implemented by the derived classes. If not
     // applicable, raise an exception
 
@@ -41,6 +46,10 @@ public:
     virtual void mul_matrix(const MatrixBase &other,
                             MatrixBase &result) const = 0;
 
+    // Matrix elementwise Multiplication
+    virtual void elementwise_mul_matrix(const MatrixBase &other,
+                                        MatrixBase &result) const = 0;
+
     // Add a scalar
     virtual void add_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const = 0;
@@ -49,9 +58,15 @@ public:
     virtual void mul_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const = 0;
 
+    // Matrix conjugate
+    virtual void conjugate(MatrixBase &result) const = 0;
+
     // Matrix transpose
     virtual void transpose(MatrixBase &result) const = 0;
 
+    // Matrix conjugate transpose
+    virtual void conjugate_transpose(MatrixBase &result) const = 0;
+
     // Extract out a submatrix
     virtual void submatrix(MatrixBase &result, unsigned row_start,
                            unsigned col_start, unsigned row_end,
@@ -114,6 +129,19 @@ public:
         return col_;
     }
 
+    virtual bool is_lower() const;
+    virtual bool is_upper() const;
+    virtual tribool is_zero() const;
+    virtual tribool is_diagonal() const;
+    virtual tribool is_real() const;
+    virtual tribool is_symmetric() const;
+    virtual tribool is_hermitian() const;
+    virtual tribool is_weakly_diagonally_dominant() const;
+    virtual tribool is_strictly_diagonally_dominant() const;
+    virtual tribool is_positive_definite() const;
+    virtual tribool is_negative_definite() const;
+
+    RCP<const Basic> trace() const;
     virtual unsigned rank() const;
     virtual RCP<const Basic> det() const;
     virtual void inv(MatrixBase &result) const;
@@ -124,6 +152,10 @@ public:
     // Matrix multiplication
     virtual void mul_matrix(const MatrixBase &other, MatrixBase &result) const;
 
+    // Matrix elementwise Multiplication
+    virtual void elementwise_mul_matrix(const MatrixBase &other,
+                                        MatrixBase &result) const;
+
     // Add a scalar
     virtual void add_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const;
@@ -132,9 +164,15 @@ public:
     virtual void mul_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const;
 
+    // Matrix conjugate
+    virtual void conjugate(MatrixBase &result) const;
+
     // Matrix transpose
     virtual void transpose(MatrixBase &result) const;
 
+    // Matrix conjugate transpose
+    virtual void conjugate_transpose(MatrixBase &result) const;
+
     // Extract out a submatrix
     virtual void submatrix(MatrixBase &result, unsigned row_start,
                            unsigned col_start, unsigned row_end,
@@ -183,9 +221,14 @@ public:
                                  const RCP<const Basic> &k, DenseMatrix &B);
     friend void mul_dense_dense(const DenseMatrix &A, const DenseMatrix &B,
                                 DenseMatrix &C);
+    friend void elementwise_mul_dense_dense(const DenseMatrix &A,
+                                            const DenseMatrix &B,
+                                            DenseMatrix &C);
     friend void mul_dense_scalar(const DenseMatrix &A,
                                  const RCP<const Basic> &k, DenseMatrix &C);
+    friend void conjugate_dense(const DenseMatrix &A, DenseMatrix &B);
     friend void transpose_dense(const DenseMatrix &A, DenseMatrix &B);
+    friend void conjugate_transpose_dense(const DenseMatrix &A, DenseMatrix &B);
     friend void submatrix_dense(const DenseMatrix &A, DenseMatrix &B,
                                 unsigned row_start, unsigned col_start,
                                 unsigned row_end, unsigned col_end,
@@ -289,6 +332,9 @@ private:
     // Stores the dimension of the Matrix
     unsigned row_;
     unsigned col_;
+
+    tribool shortcut_to_posdef() const;
+    tribool is_positive_definite_GE();
 };
 
 // ----------------------------- Sparse Matrices -----------------------------//
@@ -332,6 +378,10 @@ public:
     // Matrix Multiplication
     virtual void mul_matrix(const MatrixBase &other, MatrixBase &result) const;
 
+    // Matrix elementwise Multiplication
+    virtual void elementwise_mul_matrix(const MatrixBase &other,
+                                        MatrixBase &result) const;
+
     // Add a scalar
     virtual void add_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const;
@@ -340,9 +390,15 @@ public:
     virtual void mul_scalar(const RCP<const Basic> &k,
                             MatrixBase &result) const;
 
+    // Matrix conjugate
+    virtual void conjugate(MatrixBase &result) const;
+
     // Matrix transpose
     virtual void transpose(MatrixBase &result) const;
-    CSRMatrix transpose() const;
+    CSRMatrix transpose(bool conjugate = false) const;
+
+    // Matrix conjugate transpose
+    virtual void conjugate_transpose(MatrixBase &result) const;
 
     // Extract out a submatrix
     virtual void submatrix(MatrixBase &result, unsigned row_start,
